#include "stdafx.h"
#include "screensaver.h"

class CTestScreensaver : public CScreensaverApp
{
public:

	CTestScreensaver();
	virtual ~CTestScreensaver();

	// Accessors
	bool	isFullscreen() const		{ return isFullscreen_; }
	bool	isActive() const			{ return isActive_; }

protected:

	// Methods inherited from base class.
	// Required for fullscreen rendering.
	virtual HWND doCreateFullscreen();
	virtual void doRenderFullscreen();
	virtual void doDestroyFullscreen();
	virtual void doRestoreFullscreen();

	// Required for preview rendering.
	virtual HWND doCreatePreview( HWND hParentWnd );
	virtual void doRenderPreview();
	virtual void doDestroyPreview();
	virtual void doRestorePreview();

	// Required for configuration
	virtual void doRunConfiguration();

	// Required for registering the application
	// settings.  Typically, a company name.
	virtual LPCTSTR	applicationRegistryKey() const;

private:

	// Canonical form revoked
	CTestScreensaver( const CTestScreensaver &copyMe );
	CTestScreensaver &operator =( const CTestScreensaver &copyMe );

	// Helpers
	bool		createWindow( bool fullscreen,HWND hParentWnd );
	LPCTSTR		registerClass();
	void		renderFrame();

	// The window procedure is our friend
	friend LRESULT CALLBACK WndProc( HWND hWnd,UINT msg,WPARAM wParam,LPARAM lParam );

	// Data members
	HWND	hRenderWnd_;
	bool	isFullscreen_;
	bool	isActive_;

};

// We need an application instance
CTestScreensaver	theApp;

// Our screensaver will need a window procedure.  This is it.
LRESULT CALLBACK WndProc( HWND hWnd,UINT msg,WPARAM wParam,LPARAM lParam )
{
	switch( msg ) 
	{
		case WM_CREATE:
		{
			CTestScreensaver	*pApp=(CTestScreensaver *)AfxGetApp();
			ASSERT( pApp );

			// Hide mouse if fullscreen
			if ( pApp->isFullscreen() )
			{
				::ShowCursor( FALSE );
			}

			// Window should always on the top 
			::SetWindowPos( hWnd,HWND_TOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE );
			break;
		}
		case WM_LBUTTONDOWN:
		case WM_RBUTTONDOWN:
		case WM_MBUTTONDOWN:
		case WM_KEYDOWN:
		case WM_SYSKEYDOWN:
		{
			// Any sort of keyboard or mouse action
			// closes our screensaver.
			::SendMessage( hWnd,WM_CLOSE,0,0 );
			break;
		}
		case WM_CLOSE:
		{
			// It's all over.
			::DestroyWindow( hWnd );
			break;
		}
		case WM_DESTROY:
		{
			CTestScreensaver	*pApp=(CTestScreensaver *)AfxGetApp();
			ASSERT( pApp );

			// Ensure we are properly shutdown
			if ( pApp->isFullscreen() )
			{
				pApp->doDestroyFullscreen();
			}
			else
			{
				pApp->doDestroyPreview();
			}

			// Show mouse if we were in fullscreen mode
			if ( pApp->isFullscreen() )
			{
				::ShowCursor( TRUE );
			}

			// Ensure the app knows that the window has gone
			pApp->hRenderWnd_=NULL;
			break;
		}
		case WM_PAINT:
		{
			// Very rudimentary painting.
			static size_t	frameCounter=0;
			PAINTSTRUCT		ps;
			HDC				hDC=::BeginPaint( hWnd,&ps );
			TCHAR			szBuff[256];
			wsprintf( szBuff,_T("Frame: %u"),++frameCounter );
			::FillRect( hDC,&ps.rcPaint,(HBRUSH)::GetStockObject(BLACK_BRUSH) );
			::TextOut( hDC,0,0,szBuff,lstrlen(szBuff) );
			::EndPaint( hWnd,&ps );
			break;
		}
		default:
			return ::DefWindowProc( hWnd,msg,wParam,lParam );
   }

   return 0;
}

// Implementation
CTestScreensaver::CTestScreensaver() :
	CScreensaverApp(),
	hRenderWnd_( NULL ),
	isFullscreen_( false ),
	isActive_( false )
{
	// Ctor
}

// virtual
CTestScreensaver::~CTestScreensaver()
{
	// Dtor.
	// We should have already shut down properly
	ASSERT( hRenderWnd_==NULL );
	ASSERT( !isActive_ );
}

bool CTestScreensaver::createWindow( bool fullscreen,HWND hParentWnd )
{
	ASSERT( hRenderWnd_==NULL );
	LPCTSTR		lpszClassName=registerClass();
	const DWORD	dwStyles=(fullscreen ? WS_POPUP : WS_CHILD) | WS_VISIBLE;
	const int	x=0;
	const int	y=0;
	int			width, height;
	if ( fullscreen )
	{
		width	= ::GetSystemMetrics( SM_CXSCREEN );
		height	= ::GetSystemMetrics( SM_CYSCREEN );
	}
	else
	{
		RECT	r;
		ASSERT( hParentWnd!=NULL );
		::GetClientRect( hParentWnd,&r );
		width	= r.right;
		height	= r.bottom;
	}

	hRenderWnd_ = ::CreateWindow( lpszClassName,_T(""),dwStyles,x,y,width,height,hParentWnd,NULL,::AfxGetInstanceHandle(),NULL );

	return (hRenderWnd_!=NULL);
}

LPCTSTR CTestScreensaver::registerClass()
{
	// We must only ever register the class once
	static bool		hasBeenRegistered=false;
	static LPCTSTR	lpszClassName=_T("TestScreensaverClass");
	if ( !hasBeenRegistered )
	{
		WNDCLASSEX		wc;
		
		ZeroMemory( &wc,sizeof(wc) );
		wc.cbSize			= sizeof(wc);
		wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
		wc.lpfnWndProc		= (WNDPROC) WndProc;
		wc.cbClsExtra		= 0;
		wc.cbWndExtra		= 0;
		wc.hInstance		= ::AfxGetInstanceHandle();
		wc.hIcon			= NULL;
		wc.hCursor			= ::LoadCursor(NULL, IDC_ARROW);
		wc.hbrBackground	= NULL;
		wc.lpszMenuName		= NULL;
		wc.lpszClassName	= lpszClassName;

		ATOM	atom=::RegisterClassEx( &wc );
		ASSERT( atom!=0 );

		hasBeenRegistered=true;
	}

	return lpszClassName;
}

void CTestScreensaver::renderFrame()
{
	ASSERT( hRenderWnd_ );
	ASSERT( isActive_ );

	// Render the Direct3D or OpenGL frame here.
	// For our purposes, we simply invalidate
	// the client area and have WM_PAINT sort it
	// out.  Only cause a frame to be rendered
	// every second though.
	static clock_t	lastClock=0;
	clock_t			thisClock=clock();
	if ( thisClock-lastClock>1000 )
	{
		::InvalidateRect( hRenderWnd_,NULL,FALSE );
		::UpdateWindow( hRenderWnd_ );

		lastClock=thisClock;
	}
}

// virtual
LPCTSTR CTestScreensaver::applicationRegistryKey() const
{
	return _T("TestCompanyName");
}

// virtual
HWND CTestScreensaver::doCreateFullscreen()
{
	// Create a window for fullscreen rendering
	HWND	result=(createWindow(true,NULL) ? hRenderWnd_ : NULL);

	if ( result!=NULL )
	{
		// Now's the time to create Direct3D or OpenGL.
		// If everything is okay, set the isActive_ flag
		// to say that the sub-system is activated.
		isActive_=true;
	}

	return result;
}

// virtual
void CTestScreensaver::doRenderFullscreen()
{
	// Called to render a frame.
	renderFrame();
}

// virtual
void CTestScreensaver::doRestoreFullscreen()
{
	// Called if the device has been lost
	// e.g. after a power off on monitor
	// and should be restored.
}

// virtual
void CTestScreensaver::doDestroyFullscreen()
{
	// If we're active, shutdown any Direct3D
	// or OpenGL objects.  Do NOT destroy
	// the window - this is done automatically.
	if ( isActive_ )
	{
		isActive_=false;
	}
}

// virtual
HWND CTestScreensaver::doCreatePreview( HWND hParentWnd )
{
	// Create a window for Preview rendering
	HWND	result=(createWindow(false,hParentWnd) ? hRenderWnd_ : NULL);

	if ( result!=NULL )
	{
		// Now's the time to create Direct3D or OpenGL.
		// If everything is okay, set the isActive_ flag
		// to say that the sub-system is activated.
		isActive_=true;
	}

	return result;
}

// virtual
void CTestScreensaver::doRenderPreview()
{
	// Called to render a frame.
	renderFrame();
}

// virtual
void CTestScreensaver::doRestorePreview()
{
	// Called if the device has been lost
	// e.g. after a power off on monitor
	// and should be restored.
}

// virtual
void CTestScreensaver::doDestroyPreview()
{
	// If we're active, shutdown any Direct3D
	// or OpenGL objects.  Do NOT destroy
	// the window - this is done automatically.
	if ( isActive_ )
	{
		isActive_=false;
	}
}

// virtual
void CTestScreensaver::doRunConfiguration()
{
	// Pull up a dialog to alter your
	// screensaver's settings.
	// You must also persist them in the registry
	// yourself.
	::MessageBox( NULL,_T("This should be a configuration dialog!"),_T("TestScreensaver"),MB_OK );
}